package net.npaka.twitterclient;

import twitter4j.Twitter;
import twitter4j.TwitterException;
import twitter4j.TwitterFactory;
import twitter4j.auth.AccessToken;
import twitter4j.auth.RequestToken;
import android.app.Activity;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.view.Window;
import android.webkit.CookieManager;
import android.webkit.WebChromeClient;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.webkit.WebViewClient;

//OAuth認証を行うアクティビティ
public class OAuthActivity extends Activity {
    //Twitterのユーザー認証起動時のインテントパラメータ
    public static final String CALLBACK       ="callback";
    public static final String CONSUMER_KEY   ="consumer_key";
    public static final String CONSUMER_SECRET="consumer_secret";

    //Twitterのユーザー認証終了時のインテントパラメータ
    public static final String USER_ID     ="user_id";
    public static final String SCREEN_NAME ="screen_name";
    public static final String TOKEN       ="token";
    public static final String TOKEN_SECRET="token_secret";

    //変数
    private Handler     handler=new Handler();
    private WebView     webView;
    private Twitter     twitter;
    private String      callback;
    private String      url;
    private AccessToken accessToken;

    //アクティビティ生成時に呼ばれる
    @Override
    protected void onCreate(Bundle bundle) {
        super.onCreate(bundle);
        setResult(Activity.RESULT_CANCELED);

        //プログレスの有効化(7)
        requestWindowFeature(Window.FEATURE_PROGRESS);
        requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);

        //Webビューの生成
        webView=new WebView(this);
        WebSettings settings=webView.getSettings();
        settings.setSavePassword(false);
        webView.setWebChromeClient(webChromeClient);
        webView.setWebViewClient(webViewClient);
        setContentView(webView);

        //クッキーの無効化(11)
        CookieManager cookieManager=CookieManager.getInstance();
        cookieManager.setAcceptCookie(false);

        //パラメータの取得
        Intent intent=getIntent();
        callback=intent.getStringExtra(CALLBACK);
        String consumerKey   =intent.getStringExtra(CONSUMER_KEY);
        String consumerSecret=intent.getStringExtra(CONSUMER_SECRET);
        if (callback==null || consumerKey==null || consumerSecret==null) {
            finish();
        }

        //Twitterオブジェクトの取得
        twitter=new TwitterFactory().getInstance();
        twitter.setOAuthConsumer(consumerKey,consumerSecret);

        //Twitterのユーザー認証起動時の処理
        exePreTask();
    }

    //WebChromeクライアント
    private WebChromeClient webChromeClient=new WebChromeClient() {
        //プログレス変更時に呼ばれる
        @Override
        public void onProgressChanged(WebView view,int newProgress) {
            super.onProgressChanged(view,newProgress);
            
            //プログレスの進捗度の指定(7)
            setProgress(newProgress*100);
        }
    };

    //Webビュークライアント
    private WebViewClient webViewClient=new WebViewClient() {
        //Webページ読み込み時に呼ばれる
        @Override
        public void onLoadResource(WebView view, String url) {
            //コールバックURL遷移時の処理(9)
            if ((url!=null) && (url.startsWith(callback+"?"))) {
                //パラメータoauth_verifierの取得
                Uri uri=Uri.parse(url);
                String oAuthVerifier=uri.getQueryParameter("oauth_verifier");
                
                //Twitterのユーザー認証終了時の処理
                exePostTask(oAuthVerifier);
            } 
        }
    };
    
    //Twitterのユーザー認証起動時の処理(8)
    private void exePreTask() {
        //不定プログレスの表示(7)
        setProgressBarIndeterminateVisibility(true);
        Thread thread=new Thread(new Runnable(){public void run() {
            //認証URLの取得
            url=null;
            try {
                RequestToken requestToken=twitter.getOAuthRequestToken();
                if (requestToken!=null) {
                    url=requestToken.getAuthorizationURL();
                }
            } catch (TwitterException e) {
                TwitterClient.toast(OAuthActivity.this,e.toString());
                finish();
                return;
            }
            
            handler.post(new Runnable(){public void run(){
                //不定プログレスの非表示(7)
                setProgressBarIndeterminateVisibility(false);
                
                //Webビューの更新
                if (url!=null) {
                    webView.loadUrl(url);
                } else {
                    finish();
                }
            }});
        }});
        thread.start();
    }
    
    //Twitterのユーザー認証終了時の処理(10)
    private void exePostTask(final String oAuthVerifier) {
        //不定プログレスの表示(7)
        setProgressBarIndeterminateVisibility(true);
        
        Thread thread=new Thread(new Runnable(){public void run() {
            //アクセストークンの取得
            accessToken=null;
            if (oAuthVerifier!=null) {
                try {
                    accessToken=twitter.getOAuthAccessToken(oAuthVerifier);
                } catch (TwitterException e) {
                }
            }
            
            handler.post(new Runnable(){public void run(){
                //不定プログレスの非表示(7)
                setProgressBarIndeterminateVisibility(false);
                
                //アクティビティの戻り値指定と終了
                if (accessToken!=null) {
                    long userId=accessToken.getUserId();
                    String screenName=accessToken.getScreenName();
                    String token=accessToken.getToken();
                    String tokenSecret=accessToken.getTokenSecret();
                    Intent intent=new Intent();
                    intent.putExtra(USER_ID,userId);
                    intent.putExtra(SCREEN_NAME,screenName);
                    intent.putExtra(TOKEN,token);
                    intent.putExtra(TOKEN_SECRET,tokenSecret);
                    setResult(Activity.RESULT_OK,intent);
                }
                finish();
            }});
        }});
        thread.start();
    }
}
