package net.npaka.twitterclient;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import twitter4j.Status;
import twitter4j.Twitter;
import twitter4j.TwitterException;
import twitter4j.TwitterFactory;
import twitter4j.auth.AccessToken;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.util.DisplayMetrics;
import android.view.Gravity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

//Twitterクライアント
public class TwitterClient extends Activity {
    //コンシューマーキーとコンシューマーシークレットの設定(1)
    private final static String 
        CONSUMER_KEY   ="Consumer keyをココに記述",
        CONSUMER_SECRET="Consumer secretをココに記述",
        CALLBACK       ="Callback URLをココに記述";
    
    //定数
    private final static int 
        WC=LinearLayout.LayoutParams.WRAP_CONTENT,
        MP=LinearLayout.LayoutParams.MATCH_PARENT,
        MENU_SETUP   =0,
        MENU_UPDATE  =1,
        REQUEST_OAUTH=1;
    private final static String
        BR=System.getProperty("line.separator");
    
    //システム
    private static Handler handler=new Handler();
    private float dpi;
    
    //UI
    private ListView listView;
    private Map<String,Bitmap> icons=new HashMap<String,Bitmap>();
    private List<Status> statuses=new ArrayList<Status>();
    
    //Twitter情報
    private String token;
    private String tokenSecret;
    private boolean launch=true;;

    //アプリの初期化
    @Override
    public void onCreate(Bundle bundle) {
        super.onCreate(bundle);
        
        //プログレスの有効化(7)
        requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
        
        //DPIの取得(6)
        DisplayMetrics metrics=new DisplayMetrics();  
        getWindowManager().getDefaultDisplay().getMetrics(metrics); 
        dpi=metrics.density;
        
        //リストビューの生成
        listView=new ListView(this);
        listView.setLayoutParams(new LinearLayout.LayoutParams(MP,WC));
        listView.setAdapter(new ListAdapter());
        setContentView(listView);        
        
        //トークンの読み込み
        SharedPreferences pref=getSharedPreferences("token",MODE_PRIVATE);
        token      =pref.getString("token","");
        tokenSecret=pref.getString("tokenSecret","");
    }

    //レジューム時に呼ばれる
    @Override
    public void onResume() {
        super.onResume();
        //ステータスの更新
        if (launch) {
            updateStatuses();
            launch=false;
        }
    }
    
    //オプションメニューの生成
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        MenuItem item0=menu.add(0,MENU_SETUP,0,"設定");
        item0.setIcon(android.R.drawable.ic_menu_preferences);
        MenuItem item1=menu.add(0,MENU_UPDATE,0,"更新");
        item1.setIcon(android.R.drawable.ic_menu_upload);
        return true;
    }    

    //メニューアイテム選択イベントの処理
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id=item.getItemId();
        
        //Twitterのユーザー認証
        if (id==MENU_SETUP) {
            startOAuth();
        }
        //ステータスの更新
        else if (id==MENU_UPDATE) {
            updateStatuses();
        }
        return true;
    }
    
    //Twitterのユーザー認証(2)
    private void startOAuth() {
        Intent intent=new Intent(this,OAuthActivity.class);
        intent.putExtra(OAuthActivity.CONSUMER_KEY,CONSUMER_KEY);
        intent.putExtra(OAuthActivity.CONSUMER_SECRET,CONSUMER_SECRET);
        intent.putExtra(OAuthActivity.CALLBACK,CALLBACK);
        startActivityForResult(intent,REQUEST_OAUTH);        
    }
    
    //アクティビティ結果取得時に呼ばれる(2)
    @Override
    protected void onActivityResult(int requestCode,int resultCode,Intent data) {
        if (requestCode==REQUEST_OAUTH) {
            if (resultCode==RESULT_OK) {
                //Twitterのユーザー認証結果
                token      =data.getStringExtra(OAuthActivity.TOKEN);
                tokenSecret=data.getStringExtra(OAuthActivity.TOKEN_SECRET);
                
                //トークンの書き込み
                SharedPreferences pref=getSharedPreferences("token",MODE_PRIVATE);
                SharedPreferences.Editor editor=pref.edit();
                editor.putString("token",token);
                editor.putString("tokenSecret",tokenSecret);
                editor.commit();
                
                //ステータスの更新
                handler.post(new Runnable(){public void run(){
                    updateStatuses();
                }});
            }
        }
        super.onActivityResult(requestCode,resultCode,data);
    }
    
    //ステータスの更新(3)
    private void updateStatuses() {
        //Twitterのユーザー認証
        if (token.length()==0) {
            startOAuth();
            return;
        }
        
        //不定プログレスの表示(7)
        setProgressBarIndeterminateVisibility(true);
        
        //ステータスの更新
        Thread thread=new Thread(new Runnable(){public void run() {
            try {
                //ステータスの読み込み
                Twitter twitter=new TwitterFactory().getInstance();
                twitter.setOAuthConsumer(CONSUMER_KEY,CONSUMER_SECRET);
                twitter.setOAuthAccessToken(new AccessToken(token,tokenSecret));
                statuses=twitter.getHomeTimeline();
                icons.clear();
            
                handler.post(new Runnable(){public void run(){
                    //不定プログレスの非表示(7)
                    setProgressBarIndeterminateVisibility(false);
                    
                    //リストビューの更新(5)
                    ((BaseAdapter)listView.getAdapter()).notifyDataSetChanged();
                }});
            } catch (TwitterException e) {
                toast(TwitterClient.this,e.getMessage());
            }
        }});
        thread.start();
    }
    
    //トーストの表示
    public static void toast(final Activity activity,final String text) {
        handler.post(new Runnable() {public void run(){
            //不定プログレスの非表示(7)
            activity.setProgressBarIndeterminateVisibility(false);
            
            //トーストの表示
            Toast.makeText(activity,text,Toast.LENGTH_LONG).show();
        }});
    }
    
    //リストアダプタの生成
    public class ListAdapter extends BaseAdapter {
        //項目数の取得
        @Override
        public int getCount() {
            return statuses.size();
        }

        //項目の取得
        @Override
        public Object getItem(int pos) {
            return statuses.get(pos);
        }

        //項目IDの取得
        @Override
        public long getItemId(int pos) {
            return pos;
        }

        //ビューの取得
        @Override
        public View getView(int pos,View view,ViewGroup parent) {
            Context context=TwitterClient.this;
            Status status=statuses.get(pos);
                
            //レイアウトの生成
            if (view==null) {
                int padding=(int)(6*dpi);
                LinearLayout layout=new LinearLayout(context);
                layout.setBackgroundColor(Color.WHITE);
                layout.setPadding(padding,padding,padding,padding);
                layout.setGravity(Gravity.TOP);
                view=layout;

                //アイコン
                int size=(int)(48*dpi);
                ImageView imageView=new ImageView(context);
                imageView.setTag("icon");
                imageView.setLayoutParams(new LinearLayout.LayoutParams(size,size));
                layout.addView(imageView);
                
                //テキストの指定
                TextView textView=new TextView(context);
                textView.setTag("text");
                textView.setPadding(padding,0,padding,0);
                layout.addView(textView);
            }
                
            //ステータスの値の取得(4)
            ImageView imageView=(ImageView)view.findViewWithTag("icon");
            readIcon(imageView,status.getUser().getProfileImageURL().toString());
            TextView textView=(TextView)view.findViewWithTag("text");
            textView.setText("【"+status.getUser().getName()+"】"+BR+
                status.getText());
            return view;
        }        
    }
    
    //アイコンの読み込み
    private void readIcon(final ImageView imageView,final String url) {
        //キャッシュ内アイコンの取得
        if (icons.containsKey(url)) {
            imageView.setImageBitmap(icons.get(url));
            return;
        }
        
        //ネット上アイコンの取得
        imageView.setImageBitmap(null);
        Thread thread=new Thread(new Runnable(){public void run(){
            try {
                InputStream input=(new URL(url)).openStream();
                final Bitmap icon=BitmapFactory.decodeStream(input);
                if (icon!=null) {
                    icons.put(url,icon);
                    handler.post(new Runnable(){public void run(){
                        imageView.setImageBitmap(icon);
                    }});
                }
            } catch (Exception e) {
            }
        }});
        thread.start();
    }
}